@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
  * Free Software Foundation, Inc.
  *
  * Author: Timo Schulz, Nikos Mavrogiannopoulos
@@ -522,7 +522,7 @@ gnutls_openpgp_count_key_names (const gnutls_datum_t * cert)
 
 /**
  * gnutls_certificate_set_openpgp_keyring_file:
- * @cred: A certificate credentials structure
+ * @c: A certificate credentials structure
  * @file: filename of the keyring.
  * @format: format of keyring.
  *
@@ -535,15 +535,15 @@ gnutls_openpgp_count_key_names (const gnutls_datum_t * cert)
  *   negative error value.
  **/
 int
-gnutls_certificate_set_openpgp_keyring_file (gnutls_certificate_credentials_t
-                                             cred, const char *file,
+gnutls_certificate_set_openpgp_keyring_file (gnutls_certificate_credentials_t c,
+					     const char *file,
                                              gnutls_openpgp_crt_fmt_t format)
 {
   gnutls_datum_t ring;
   size_t size;
   int rc;
 
-  if (!cred || !file)
+  if (!c || !file)
     {
       gnutls_assert ();
       return GNUTLS_E_INVALID_REQUEST;
@@ -558,7 +558,7 @@ gnutls_certificate_set_openpgp_keyring_file (gnutls_certificate_credentials_t
     }
 
   rc =
-    gnutls_certificate_set_openpgp_keyring_mem (cred, ring.data, ring.size,
+    gnutls_certificate_set_openpgp_keyring_mem (c, ring.data, ring.size,
                                                 format);
 
   free (ring.data);
